Module: ActionAI::Parameterized::ClassMethods

Defined in:
lib/action_ai/parameterized.rb

Instance Method Summary collapse

Instance Method Details

#with(params) ⇒ Object

Provide the parameters to the agent in order to use them in the instance methods and callbacks.

InvitationAgent.with(inviter: person_a, invitee: person_b)..later

See Parameterized documentation for full example.



73
74
75
# File 'lib/action_ai/parameterized.rb', line 73

def with(params)
  ActionAI::Parameterized::Agent.new(self, params)
end