Module: ActionAI::ImplicitInteraction
- Included in:
- Agent
- Defined in:
- lib/action_ai/implicit_interaction.rb
Overview
Action AI Implicit Interaction
Handles implicit interactions for an agent action that does not explicitly
respond with ask.
If no explicit ask is performed by the action, the implicit response is
to call ask with the default prompt rendering.
Instance Method Summary collapse
Instance Method Details
#default_interaction ⇒ Object
17 |
# File 'lib/action_ai/implicit_interaction.rb', line 17 def default_interaction = ask |
#send_action ⇒ Object
12 13 14 15 |
# File 'lib/action_ai/implicit_interaction.rb', line 12 def send_action(...) super .tap { default_interaction unless performed? } end |